@@ -260,12 +260,14 @@
 
 			resetToggle();
 
-			// Clear password field to prevent update
-			$pass1.val( '' ).trigger( 'pwupdate' );
-			$submitButtons.prop( 'disabled', false );
+			if ( $pass1Row.closest( 'form' ).is( '#your-profile' ) ) {
+				// Clear password field to prevent update
+				$pass1.val( '' ).trigger( 'pwupdate' );
+				$submitButtons.prop( 'disabled', false );
+			}
 		} );
 
-		$pass1Row.closest('form').on( 'submit', function () {
+		$pass1Row.closest( 'form' ).on( 'submit', function () {
 			updateLock = false;
 
 			$pass1.prop( 'disabled', false );
